#!/bin/bash

if [ -d "scenario10" -o -d "scenario20" ]
then
    echo "scenario10 or scenario20 already exist."
    exit 1
fi
#Create directories: scenario.xml for all routing agents
#scenario10 for 10000 nodes and scenario20 for 20000 nodes.
for scen in scenario10 scenario20; do
    for rout in Backoff_Flooding SLR; do
	for ring in NoRing Ring; do
	    par="${rout}_${ring}"
	    for rng in `seq 0 9`; do
		dir="${scen}/${par}/run${rng}"
		mkdir -p $dir
		cp scenario.xml ${dir}/scenario.xml
		if [ $par = "Backoff_Flooding_NoRing" ]
		then
		    sed -i 's/routing defaultBackoffWindow="10000" backoffRNGSeed="1"/routing/g' ${dir}/scenario.xml
		    sed -i 's/PureFloodingRouting/BackoffFloodingRouting/g' ${dir}/scenario.xml
		    sed -i 's/DEDeN enabled="false"/DEDeN enabled="true"/g' ${dir}/scenario.xml
		elif [ $par = "Backoff_Flooding_Ring" ]
		then
		    sed -i 's/routing defaultBackoffWindow="10000" backoffRNGSeed="1"/routing/g' ${dir}/scenario.xml
		    sed -i 's/PureFloodingRouting/BackoffFloodingRingRouting rangeBig_nm="900000" rangeSmall_nm="800000"/g' ${dir}/scenario.xml
		    sed -i 's/DEDeN enabled="false"/DEDeN enabled="true"/g' ${dir}/scenario.xml
		elif [ $par = "SLR_NoRing" ]
		then
		    sed -i 's/PureFloodingRouting/SLRRouting commRangeSetup_nm="450000" anchor1id="0" anchor2id="1"/g' ${dir}/scenario.xml
		elif [ $par = "SLR_Ring" ]
		then
		    sed -i 's/PureFloodingRouting/SLRRingRouting commRangeSetup_nm="450000" anchor1id="0" anchor2id="1" rangeBig_nm="900000" rangeSmall_nm="800000"/g' ${dir}/scenario.xml
		fi
		if [ $scen = "scenario20" ]
		then
		    sed -i 's/count="9996"/count="19996"/g' ${dir}/scenario.xml
		fi
	    done
	done
    done
done

#Run bitsimulator

for scen in scenario10 scenario20; do
    for rout in Backoff_Flooding SLR; do
        for ring in NoRing Ring; do
	    par="${rout}_${ring}"
	    #beginpar
	    for rng in `seq 0 9`; do
		dir="${scen}/${par}/run${rng}"
		echo Running ${scen} ${par} with seed ${rng}
		time ./bitsimulator -D ${dir} --genericNodesRNGSeed ${rng} --doNotUseNeighboursList &> /dev/null
		awk '$1 == "1" &&  $6=="1003"'  ${dir}/events.log > ${dir}/receivedata.log
		awk '$1 == "0" &&  $7=="0"'  ${dir}/events.log > ${dir}/senddata.log
		awk '$1 == "0"'  ${dir}/events.log > ${dir}/allsend.log
		rm ${dir}/events.log
	    done
	    #endpar
	done
    done
done
#waitpar

for scen in scenario10; do
    for rout in Backoff_Flooding SLR; do
        for ring in NoRing Ring; do
	    par="${rout}_${ring}"
	    if [ $rout = "SLR" ]
	    then
		echo "seed packet_sequence_number sending_nodes_per_packet destination_reached transmission_time_per_packet" > ${scen}/${par}/results.txt
		echo "seed time_destination_reached_packet0 1 2 3 4 5 6 7 8 9" > ${scen}/${par}/receiver.txt
	    else
		echo "seed packet_sequence_number sending_nodes_per_packet receiving_nodes_per_packet transmission_time_per_packet" > ${scen}/${par}/results.txt
	        echo "seed time_packet0 receiving_nodes_packet0 t4 rx4 t5 rx5 t8 rx8 t9 rx9" > ${scen}/${par}/receiver.txt
	    fi
	    echo "seed time sending_nodes_packet0" > ${scen}/${par}/forwarder.txt
	    echo seed number_control1_packets number_control2_packets number_data_packets > ${scen}/${par}/control.txt
	    #beginpar
	    for rng in `seq 0 9`; do
		dir="${scen}/${par}/run${rng}"
		sendcontrol1=`awk '$7=="9"'  ${dir}/allsend.log | wc -l`
		sendcontrol2=`awk '$7=="10"'  ${dir}/allsend.log | wc -l`
		senddata=`awk '$7=="0"'  ${dir}/allsend.log | wc -l`
		echo ${rng} ${sendcontrol1} ${sendcontrol2} ${senddata} >> ${scen}/${par}/control.txt
		for p in `seq 0 9`
		do
		    send=`awk -v P="$p" '$9==P' ${dir}/senddata.log | sort -k3,3 -u | wc -l`
		    sendt=`awk -v P="$p" '$3 == "2" &&  $9==P' ${dir}/senddata.log | awk 'NR==1{print $2; exit}'`
		    if [ $rout = "SLR" ]
		    then
			receive=`awk -v P="$p" '$3=="3" &&  $8==P' ${dir}/receivedata.log | sort -k3,3 -u | wc -l`
			receivet=`awk -v P="$p" '$3=="3" &&  $8==P' ${dir}/receivedata.log | sort -k3,3 -u | sort -k2,2 | awk 'END{print $2; exit}'`
		    else
			receive=`awk -v P="$p" '$8==P' ${dir}/receivedata.log | sort -k3,3 -u | wc -l`
			receivet=`awk -v P="$p" '$8==P' ${dir}/receivedata.log | sort -k3,3 -u | sort -k2,2 | awk 'END{print $2; exit}'`
		    fi
		    echo ${rng} ${p} ${send} ${receive} $(expr $receivet - $sendt) >>  ${scen}/${par}/results.txt
		done
		if [ $rout = "SLR" ]
		then
		    receivet0=`awk '$3=="3" && $8=="0"' ${dir}/receivedata.log | sort -k3,3 -u | awk 'END{print $2; exit}'`
		    receivet1=`awk '$3=="3" && $8=="1"' ${dir}/receivedata.log | sort -k3,3 -u | awk 'END{print $2; exit}'`
		    receivet2=`awk '$3=="3" && $8=="2"' ${dir}/receivedata.log | sort -k3,3 -u | awk 'END{print $2; exit}'`
		    receivet3=`awk '$3=="3" && $8=="3"' ${dir}/receivedata.log | sort -k3,3 -u | awk 'END{print $2; exit}'`
		    receivet4=`awk '$3=="3" && $8=="4"' ${dir}/receivedata.log | sort -k3,3 -u | awk 'END{print $2; exit}'`
		    receivet5=`awk '$3=="3" && $8=="5"' ${dir}/receivedata.log | sort -k3,3 -u | awk 'END{print $2; exit}'`
		    receivet6=`awk '$3=="3" && $8=="6"' ${dir}/receivedata.log | sort -k3,3 -u | awk 'END{print $2; exit}'`
		    receivet7=`awk '$3=="3" && $8=="7"' ${dir}/receivedata.log | sort -k3,3 -u | awk 'END{print $2; exit}'`
		    receivet8=`awk '$3=="3" && $8=="8"' ${dir}/receivedata.log | sort -k3,3 -u | awk 'END{print $2; exit}'`
		    receivet9=`awk '$3=="3" && $8=="9"' ${dir}/receivedata.log | sort -k3,3 -u | awk 'END{print $2; exit}'`
		    echo ${rng} ${receivet0} ${receivet1} ${receivet2} ${receivet3} ${receivet4} ${receivet5} ${receivet6} ${receivet7} ${receivet8} ${receivet9}  >> ${scen}/${par}/receiver.txt
		fi
	    done
	    for rng in 0; do
		dir="${scen}/${par}/run${rng}"
		if [ $rout != "SLR" ]
		then
		    for d in 0 4 5 8 9; do #l is time receive
			for o in `seq 0 50 10000`; do
		            l=$((1000000 * $o + 12000000000000 + 1000000000000 * $d))  #intervalstep + start time packet0 4 5 8 9
			    numberreceive0=`awk -v ti="${l}" '$2 < ti &&  $8=="0"' ${dir}/receivedata.log  | sort -k3,3 -u | wc -l`
			    numberreceive4=`awk -v ti="${l}" '$2 < ti &&  $8=="4"' ${dir}/receivedata.log  | sort -k3,3 -u | wc -l`
			    numberreceive5=`awk -v ti="${l}" '$2 < ti &&  $8=="5"' ${dir}/receivedata.log  | sort -k3,3 -u | wc -l`
			    numberreceive8=`awk -v ti="${l}" '$2 < ti &&  $8=="8"' ${dir}/receivedata.log  | sort -k3,3 -u | wc -l`
			    numberreceive9=`awk -v ti="${l}" '$2 < ti &&  $8=="9"' ${dir}/receivedata.log  | sort -k3,3 -u | wc -l`
			    echo ${rng} ${l} ${numberreceive0} ${numberreceive4} ${numberreceive5} ${numberreceive8} ${numberreceive9} >> ${scen}/${par}/receiver.txt
			done
		    done
		fi
		for o in `seq 0 50 13000`; do
		    l=$((1000000 * $o + 12000000000000)) #intervalstep + start time packet0
		    numbersend0=`awk -v ti="${l}" '$2 < ti && $9=="0"' ${dir}/senddata.log  | sort -k3,3 -u | wc -l`
		    echo ${rng} ${l} ${numbersend0} >>  ${scen}/${par}/forwarder.txt
		done
	    done
	    awk1=`awk '{s+=$3} END{print s/(NR - 1)}'  ${scen}/${par}/results.txt`
	    awk2=`awk '{t+=$4} END{print t/(NR - 1)}'  ${scen}/${par}/results.txt`
	    awk3=`awk '{q+=$5} END{print q/(NR - 1)}'  ${scen}/${par}/results.txt`
	    echo averages: ${awk1} ${awk2} ${awk3} >>  ${scen}/${par}/results.txt
	    awk4=`awk '{a+=$2} END{print a/(NR - 1)}'  ${scen}/${par}/control.txt`
	    awk5=`awk '{b+=$3} END{print b/(NR - 1)}'  ${scen}/${par}/control.txt`
	    awk6=`awk '{c+=$4} END{print c/(NR - 1)}'  ${scen}/${par}/control.txt`
	    echo averages: ${awk4} ${awk5} ${awk6} >>  ${scen}/${par}/control.txt
	    #endpar
	done
    done
done
#waitpar

#prepare data to plot: use receiver2.txt and forwarder2.txt for plotting
for scen in scenario10; do
    for rout in Backoff_Flooding SLR; do
	for ring in NoRing Ring; do
	    par="${rout}_${ring}"
	    if [ $rout = "SLR" ]
	    then
		sed '1d'  ${scen}/${par}/receiver.txt | awk '{print ($2-12000000000000)/1000000, ($3-12000000000000)/1000000, ($4-12000000000000)/1000000, ($5-12000000000000)/1000000, ($6-12000000000000)/1000000, ($7-12000000000000)/1000000, ($8-12000000000000)/1000000, ($9-12000000000000)/1000000, ($10-12000000000000)/1000000, ($11-12000000000000)/1000000}' | awk '
{ 
    for (i=1; i<=NF; i++)  {
        a[NR,i] = $i
    }
}
NF>p { p = NF }
END {    
    for(j=1; j<=p; j++) {
        str=a[1,j]
        for(i=2; i<=NR; i++){
            str=str" "a[i,j];
        }
        print str
    }
}' >  ${scen}/${par}/receiver2.txt
	    else
		awk '$2 < "13000000000000"  && $2  > "12000000000000" '  ${scen}/${par}/receiver.txt | awk 'FNR > 1 { print ($2-12000000000000)/1000000, $3}' > ${scen}/${par}/receiver0.txt
		awk '$2 < "17000000000000" && $2  > "16000000000000" '  ${scen}/${par}/receiver.txt | awk 'FNR > 1 { print ($2-16000000000000)/1000000, $4}' > ${scen}/${par}/receiver4.txt
		awk '$2 < "18000000000000" && $2  > "17000000000000" '  ${scen}/${par}/receiver.txt | awk 'FNR > 1 { print ($2-17000000000000)/1000000, $5}' > ${scen}/${par}/receiver5.txt
		awk '$2 < "21000000000000" && $2  > "20000000000000" '  ${scen}/${par}/receiver.txt | awk 'FNR > 1 { print ($2-20000000000000)/1000000, $6}' > ${scen}/${par}/receiver8.txt
	    fi
	     awk 'FNR > 1 { print ($2-12000000000000)/1000000, $3}' ${scen}/${par}/forwarder.txt  >  ${scen}/${par}/forwarder2.txt
	done
    done
done

#for scenario with 20000 nodes, we only need results.txt
for scen in scenario20; do
    for rout in Backoff_Flooding SLR; do
        for ring in NoRing Ring; do
	    par="${rout}_${ring}"
	    if [ $rout = "SLR" ]
	    then
		echo "seed packet_sequence_number sending_nodes_per_packet destination_reached transmission_time_per_packet" > ${scen}/${par}/results.txt
	    else
		echo "seed packet_sequence_number sending_nodes_per_packet receiving_nodes_per_packet transmission_time_per_packet" > ${scen}/${par}/results.txt
	    fi
	    #beginpar
	    for rng in `seq 0 9`; do
		dir="${scen}/${par}/run${rng}"
		for p in `seq 0 9`
		do
		    send=`awk -v P="$p" '$9==P' ${dir}/senddata.log | sort -k3,3 -u | wc -l`
		    sendt=`awk -v P="$p" '$3 == "2" &&  $9==P' ${dir}/senddata.log | awk 'NR==1{print $2; exit}'`
		    if [ $rout = "SLR" ]
		    then
			receive=`awk -v P="$p" '$3=="3" &&  $8==P' ${dir}/receivedata.log | sort -k3,3 -u | wc -l`
			receivet=`awk -v P="$p" '$3=="3" &&  $8==P' ${dir}/receivedata.log | sort -k3,3 -u | sort -k2,2 | awk 'END{print $2; exit}'`
		    else
			receive=`awk -v P="$p" '$8==P' ${dir}/receivedata.log | sort -k3,3 -u | wc -l`
			receivet=`awk -v P="$p" '$8==P' ${dir}/receivedata.log | sort -k3,3 -u | sort -k2,2 | awk 'END{print $2; exit}'`
		    fi
		    echo ${rng} ${p} ${send} ${receive} $(expr $receivet - $sendt) >>  ${scen}/${par}/results.txt
		done
	    done
	    awk1=`awk '{s+=$3} END{print s/(NR - 1)}' ${scen}/${par}/results.txt`
	    awk2=`awk '{t+=$4} END{print t/(NR - 1)}' ${scen}/${par}/results.txt`
	    awk3=`awk '{q+=$5} END{print q/(NR - 1)}' ${scen}/${par}/results.txt`
	    echo averages: ${awk1} ${awk2} ${awk3} >> ${scen}/${par}/results.txt
	    #endpar
	done
    done
done
#waitpar

gnuplot *.gnuplot
